import { reactive } from 'vue-demi';
class FormInstance {
    initialData;
    formData;
    rules;
    options;
    dirtyFields = new Set();
    formErrors = [];
    $form;
    $fields;
    constructor(data, rules, options = {}) {
        this.initialData = reactive({ ...data });
        this.formData = reactive(data);
        this.rules = rules;
        this.options = options;
        this.$fields = {};
        this.$form = {};
        this.setupForm();
        this.setupFields();
    }
    setupForm() {
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const self = this;
        const keys = Object.keys(this.formData);
        this.$form = {
            get $dirty() {
                return Object.values(self.$fields).some(field => field.$dirty);
            },
            get $invalid() {
                return Object.values(self.$fields).some(field => field.$invalid);
            },
            get $errors() {
                return self.formErrors;
            },
            $validate: () => {
                return keys
                    .map(field => this.$fields[field].$validate())
                    .every(result => result);
            },
            $reset: () => {
                keys.forEach(field => {
                    this.$fields[field].$reset();
                });
            },
            $update: (newData) => {
                Object.assign(this.formData, newData);
                Object.assign(this.initialData, newData);
            },
            $touch: () => {
                keys.forEach(field => {
                    this.$fields[field].$touch();
                });
            },
        };
    }
    updateFormErrors() {
        this.formErrors.length = 0;
        Object.values(this.$fields).forEach(field => {
            this.formErrors.push(...field.$errors);
        });
    }
    setupFields() {
        Object.keys(this.formData).forEach(field => {
            this.$fields[field] = this.createFieldState(field);
        });
    }
    createFieldState(field) {
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const self = this;
        const fieldState = reactive({
            $invalid: false,
            $dirty: false,
            $errors: [],
            $touch: () => self.markAsDirty(field),
            $reset: () => {
                self.formData[field] = self.initialData[field];
                self.dirtyFields.delete(field);
                fieldState.$dirty = false;
                fieldState.$errors = [];
                fieldState.$invalid = false;
            },
            $update: (value) => {
                self.formData[field] = value;
                if (!self.options.lazy)
                    self.validateField(field);
            },
            $validate: () => self.validateField(field),
            get value() {
                return self.formData[field];
            },
            set value(val) {
                self.formData[field] = val;
                if (!self.options.lazy)
                    self.validateField(field);
                if (self.options.autoTouch)
                    self.markAsDirty(field);
            },
        });
        return fieldState;
    }
    markAsDirty(field) {
        if (!this.dirtyFields.has(field)) {
            this.dirtyFields.add(field);
            this.$fields[field].$dirty = true;
        }
    }
    validateField(field) {
        const value = this.formData[field];
        const fieldEntry = this.$fields[field];
        fieldEntry.$errors = [];
        this.rules[field].forEach(rule => {
            if (!rule.test(value)) {
                const message = typeof rule.message === 'function'
                    ? rule.message(value)
                    : rule.message;
                fieldEntry.$errors.push(`${rule.name}: ${message}`);
            }
        });
        fieldEntry.$invalid = fieldEntry.$errors.length > 0;
        this.updateFormErrors();
        return !fieldEntry.$invalid;
    }
}
export class FormManager {
    buildForm(data, rules, options) {
        return new FormInstance(data, rules, options);
    }
}
//# sourceMappingURL=vue-form-manager.js.map